<?php

App::uses('Component', 'Controller');

class ImpersonateComponent extends Component {

	protected $_controller = null;

	public $defaultSettings = [
		// Define the model used by the Auth component.
		'userModel' => [
			'className' => 'EvCore.User',
			'alias' => 'User',
		],
		// Define any additional parameters for the query used to retrieve the
		// user.
		'params' => [
			'conditions' => [
				'User.user_group_id >' => 2
			]
		],
		// Define the query parameters for checking the current user can
		// impersonate other users.
		'canImpersonateParams' => [
			'conditions' => [
				'User.user_group_id <=' => 2
			]
		],
		//Define the permissions path for checking if the current user can impersonate
		//other users.
		'canImpersonatePermission' => 'impersonations',
		// Define the redirect route/url to go to after starting to impersonate
		// a user.
		'impersonateRedirect' => '/',
		// Define the redirect route/url to go to after ending an impersonation
		// session.
		'unimpersonateRedirect' => [
			'admin' => true,
			'plugin' => 'ev_core',
			'controller' => 'users',
			'action' => 'index'
		],
		//Define whether the unimpersonate method should logout the current user
		//if there is currently nobody impersonating them.
		'unimpersonateLogout' => true,
	];

	public function initialize(Controller $controller) {
		parent::initialize($controller);
		$this->_controller = $controller;
		$this->settings = Hash::merge($this->defaultSettings, $this->settings);
	}

/**
 * Impersonate another user.
 *
 * @param int   $userId ID of the user we want to impersonate.
 * @param array $params Additional query parameters for user to impersonate.
 * @return void|bool Redirects if setting redirects are set. Otherwise true/false depending on impersonation success.
 */
	public function impersonate($userId, array $params = []) {
		// Make sure we are not already impersonating a user before
		// impersonating someone.
		if ($this->isImpersonating()) {
			$this->unimpersonate();
		}

		$impersonator = $this->_controller->Auth->user();
		if (
			!$this->isImpersonating() &&
			$this->_canImpersonate($impersonator) === true
		) {
			// Get the user to impersonate.
			$this->_controller->loadModel($this->settings['userModel']['className']);
			if (!empty($this->settings['params'])) {
				$params = Hash::merge($this->settings['params'], $params);
			}
			$defaultParams = [
				'conditions' => [
					$this->_controller->{$this->settings['userModel']['alias']}->escapeField() => $userId
				]
			];
			$user = $this->_controller->{$this->settings['userModel']['alias']}->find('first', Hash::merge($defaultParams, $params));

			if (!empty($user)) {
				// Store the current auth user as the impersonator.
				$this->_controller->Session->write('Auth.Impersonator', $impersonator);

				$fullUser = $this->_controller->{$this->settings['userModel']['alias']}->getUserForLogin($user['User']['id']);

				// Change to recognised auth user to the impersonated user.
				$this->_controller->Auth->login($fullUser);

				// Redirect to ensure we are in the impersonated users session.
				if (!empty($this->settings['impersonateRedirect'])) {
					return $this->_controller->redirect($this->settings['impersonateRedirect']);
				} else {
					return true;
				}
			}
		}

		$this->_controller->Session->setFlash('Unable to impersonate user!', 'flash_fail');
		if (!empty($this->settings['unimpersonateRedirect'])) {
			return $this->_controller->redirect($this->settings['unimpersonateRedirect']);
		}

		return false;
	}

/**
 * Stop impersonating a user and return to the original session.
 *
 * @return void|bool Redirects the user if setting redirects are set. Otheriwse true/false depeding on unimpersonation success.
 */
	public function unimpersonate() {
		if ($this->isImpersonating()) {
			// Restore the impersonator's login as the current user.
			$user = $this->getImpersonator();
			$currentUser = $this->_controller->Auth->user();

			//Login the impersonating user back in.
			if ($user['User']['id'] != $currentUser['User']['id']) {
				$originalLoggedIn = $this->_controller->Auth->login($user);
			}

			$this->removeImpersonator();

			if (!empty($this->settings['unimpersonateRedirect'])) {
				return $this->_controller->redirect($this->settings['unimpersonateRedirect']);
			} else {
				return $originalLoggedIn;
			}
		}

		if ($this->settings['unimpersonateLogout']) {
			return $this->_controller->Auth->logout();
		}

		return null;
	}

/**
 * Check if an impersonater is allowed to impersonate another user.
 *
 * @param array $impersonator User data from the auth session.
 * @return bool
 */
	public function canImpersonate($impersonator) {
		return $this->_canImpersonate($impersonator);
	}

/**
 * Check if the impersonator is allowed to impersonate other users.
 *
 * @param array $impersonator User data from the auth session.
 * @return bool
 */
	protected function _canImpersonate($impersonator) {
		if (!empty(Configure::read('EvImpersonate.useAclPermissions'))) {
			return $this->_canImpersonatePermission($impersonator);
		} else {
			return $this->_canImpersonateUserLevel($impersonator);
		}
	}

/**
 * Check if a user can impersonate other users by checking their user data.
 *
 * @param array $impersonator User data from the auth session.
 * @return bool
 */
	protected function _canImpersonateUserLevel($impersonator) {
		$primaryKey = $this->_controller->{$this->settings['userModel']['alias']}->primaryKey;
		$alias = $this->_controller->{$this->settings['userModel']['alias']}->alias;
		$defaultParams = [
			'conditions' => [
				$this->_controller->{$this->settings['userModel']['alias']}->escapeField() => $impersonator[$alias][$primaryKey]
			]
		];
		return (bool)$this->_controller->{$this->settings['userModel']['alias']}->find(
			'first',
			Hash::merge($defaultParams, $this->settings['canImpersonateParams'])
		);
	}

/**
 * Check if a user cna impersonate other users by checking their permissions.
 *
 * @param array $impersonator User data from the auth session.
 * @return bool
 */
	protected function _canImpersonatePermission($impersonator) {
		if (empty($impersonator['User']['user_group_id'])) {
			return false;
		}

		return $this->_controller->Acl->check(
			[
				'model' => 'UserGroup',
				'foreign_key' => $impersonator['User']['user_group_id']
			],
			$this->settings['canImpersonatePermission']
		);
	}

/**
 * Check if the current user is impersonating another user by checking the session data.
 *
 * @return bool.
 */
	public function isImpersonating() {
		return $this->_controller->Session->check('Auth.Impersonator') === true;
	}

/**
 * Get the current impersonator.
 *
 * @return array Impersonator data from the auth session.
 */
	public function getImpersonator() {
		return $this->_controller->Session->read('Auth.Impersonator');
	}

/**
 * Remove the current impersonator from the session. This is normally handled by the unimpersonate
 * method but this method can be used separately to tidy up the session data to stop the user
 * impersonating themself.
 *
 * @return bool. True if removed, otherwise false.
 */
	public function removeImpersonator() {
		return $this->_controller->Session->delete('Auth.Impersonator');
	}

}
